package com.marvinelsen.willow.ui.dialogs

import com.marvinelsen.willow.Config
import com.marvinelsen.willow.WillowApplication
import com.marvinelsen.willow.ui.Pronunciation
import com.marvinelsen.willow.ui.formatters.FontSizeTextFormatter
import javafx.fxml.FXMLLoader
import javafx.scene.control.ButtonType
import javafx.scene.control.CheckBox
import javafx.scene.control.ComboBox
import javafx.scene.control.Dialog
import javafx.scene.control.DialogPane
import javafx.scene.control.Spinner
import javafx.stage.Modality
import javafx.stage.Stage
import javafx.stage.Window
import javafx.util.Callback

@Suppress("MemberVisibilityCanBePrivate")
class PreferencesDialog(owner: Window?, config: Config) : Dialog<PreferencesDialog.Result>() {
    companion object {
        private const val DIALOG_MIN_HEIGHT = 400.0
        private const val DIALOG_MIN_WIDTH = 400.0
    }

    enum class Result {
        CHANGES, NO_CHANGES
    }

    lateinit var comboBoxEntryPronunciation: ComboBox<Pronunciation>
    lateinit var comboBoxListPronunciation: ComboBox<Pronunciation>

    lateinit var checkBoxListShowPronunciation: CheckBox
    lateinit var checkBoxListShowDefinition: CheckBox

    lateinit var spinnerEntryHeadwordFontSize: Spinner<Int>
    lateinit var spinnerEntryPronunciationFontSize: Spinner<Int>
    lateinit var spinnerListHeadwordFontSize: Spinner<Int>
    lateinit var spinnerListPronunciationFontSize: Spinner<Int>
    lateinit var spinnerListDefinitionFontSize: Spinner<Int>

    private val entryHeadwordFontSizeObjectProperty = config.detailHeadwordFontSize.asObject()

    init {
        val loader = FXMLLoader(WillowApplication::class.java.getResource("/fxml/preferences-dialog.fxml"))
        loader.setController(this)
        val root: DialogPane = loader.load()

        dialogPane = root
        title = "Settings"
        isResizable = true

        initOwner(owner)
        initModality(Modality.APPLICATION_MODAL)

        (dialogPane.scene.window as Stage).apply {
            minWidth = DIALOG_MIN_WIDTH
            minHeight = DIALOG_MIN_HEIGHT
        }

        with(spinnerEntryHeadwordFontSize) {
            editor.textFormatter = FontSizeTextFormatter()
            valueFactory.valueProperty().bindBidirectional(entryHeadwordFontSizeObjectProperty)
        }

        resultConverter = Callback(::convertToResult)
    }

    private fun convertToResult(buttonType: ButtonType) =
        when (buttonType) {
            ButtonType.APPLY -> Result.CHANGES
            else -> Result.NO_CHANGES
        }
}
